%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Please cite as follows:
%Eric Sims and Jing Cynthia Wu "Evaluating Central Banks' Tool Kit: Past, 
%Present, and Future", Journal of Monetary Economics, forthcoming.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

warning off;

var C L Lam mu w RF RB Q QB f b re n lam omega Omega theta phi d Pi pr x1 x2
    vp pm Ym Y K A fm Ih v1 v2 pk I RTR Rre Rd fcb bcb G T xi welf epsi;

varexo eA eG et ef eb er exi em;

parameters beta Pis RTRs bh Kai kappa X sigma thetas Delta phip gammap alpha
           delta Psi kI rhor phipi phiy fcbs bcbs rhof bG Gs rhoG As rhoA xis rhoxi
           sr sf sb sA sG st sxi eta rhob rhot Ys epsis sp phis Ls gy fracf fracb bgy lev
           Cs Lams mus ws RFs RBs Qs QBs fs bs res ns lams omegas Omegas ds 
           prs x1s x2s vps pms Yms Ks fms Ihs v1s v2s pks Is Rres Rds Ts welfs;

load param;


model;


%%%%%%%%%%%%%%%%% Household %%%%%%%%%%%%%%%%%
% (1) marginal utility
exp(mu)=(exp(C)-bh*exp(C(-1)))^(-1)-bh*beta/(exp(C(+1))-bh*exp(C));

% (2) Lam
exp(Lam)=beta*exp(mu)/exp(mu(-1));

% (3) Euler equation
1=exp(Rd)*exp(Lam(+1))/exp(Pi(+1));

% (4) labor supply
Kai*exp(eta*L)=exp(mu)*exp(w);

%%%%%%%%%%%%%%%%%%%% Financial intermediaries %%%%%%%%%%%%%%%%%%%
% (5) RF
exp(RF)=(1+kappa*exp(xi)*exp(Q))/(exp(xi(-1))*exp(Q(-1)));

% (6) RB
exp(RB)=(1+kappa*exp(xi)*exp(QB))/(exp(xi(-1))*exp(QB(-1)));

% (7) Balance sheet
exp(xi)*exp(Q)*exp(f)+Delta*exp(xi)*exp(QB)*exp(b)+exp(re)=exp(d)+exp(n);

% (8) Evolution of net worth
exp(n)*exp(Pi)=sigma*((exp(RF)-exp(Rd(-1)))*exp(xi(-1))*exp(Q(-1))*exp(f(-1))   +     (exp(RB)-exp(Rd(-1)))*exp(xi(-1))*exp(QB(-1))*exp(b(-1))  
+    (exp(Rre(-1))-exp(Rd(-1)))*exp(re(-1))  +exp(Rd(-1))*exp(n(-1))) +X;

%exp(n(+1))*exp(Pi(+1))=sigma*((exp(RF(+1))-exp(Rd))*exp(xi)*exp(Q)*exp(f)   +     (exp(RB(+1))-exp(Rd))*exp(xi)*exp(QB)*exp(b)  
%+    (exp(Rre)-exp(Rd))*exp(re)  +exp(Rd)*exp(n)) +X;


% (9) Omega
exp(Omega)=1-sigma+sigma*exp(theta)*exp(phi);

% (10) FOC for f
exp(Lam(+1))*exp(Omega(+1))/exp(Pi(+1))*(exp(RF(+1))-exp(Rd))=exp(lam)*exp(theta)/(1+exp(lam));

% (11) FOC for b
exp(Lam(+1))*exp(Omega(+1))/exp(Pi(+1))*(exp(RB(+1))-exp(Rd))=Delta*exp(lam)*exp(theta)/(1+exp(lam));

% (12) FOC for re
exp(Lam(+1))*exp(Omega(+1))/exp(Pi(+1))*(exp(Rre)-exp(Rd))=-omega/(1+exp(lam));

% (13) Endogenous leverage
exp(theta)*exp(phi)=(1+exp(lam))*exp(Lam(+1))*exp(Omega(+1))/exp(Pi(+1))*exp(Rd)-omega*exp(re)/exp(n);

% (14) Financial constraint
exp(phi)=(exp(xi)*exp(Q)*exp(f)+Delta*exp(QB)*exp(b))/exp(n);

%%%%%%%%%%%%%%%%%%% Calvo pricing %%%%%%%%%%%%%%%%%%%%%%%%%
% (15) reset price
exp(pr)=epsi/(epsi-1)*exp(x1)/exp(x2);

% (16) Auxiliary price-setting x1
exp(x1)=exp(pm)*exp(Y)+phip*exp(Lam(+1))*(exp(Pi(+1))/exp(Pi*gammap))^epsi*exp(x1(+1));

% (17) Auxiliary price-setting x2
exp(x2)=exp(Y)+phip*exp(Lam(+1))*(exp(Pi(+1))/exp(Pi*gammap))^(epsi-1)*exp(x2);

% (18) Y and Ym
exp(vp)*exp(Y)=exp(Ym);

% (19) price dispersion
exp(vp)=(1-phip)*exp((-epsi)*pr)+phip*(exp(Pi)/exp(gammap*(Pi(-1))))^epsi*exp(vp(-1));

% (20) price evolution
1=(1-phip)*exp((1-epsi)*pr)+phip*exp(gammap*(1-epsi)*Pi(-1))*exp(Pi*(epsi-1));

%%%%%%%%%%%%%%%% Firm %%%%%%%%%%%%%%%%%%%%%%
% (21) Production function
exp(Ym)=exp(A)*exp(K(-1)*alpha)*exp(L*(1-alpha));

% (22) Capital accumulation
exp(K)=exp(Ih)+(1-delta)*exp(K(-1));

% (23) financial constraint
Psi*exp(pk)*exp(Ih)=exp(xi)*exp(Q)*(exp(fm)-kappa*exp(fm(-1))/exp(Pi));

% (24) FOC for capital
exp(Lam(+1))*(alpha*exp(pm(+1))*exp(Ym(+1))/exp(K)+exp(v1(+1))*(1-delta))=exp(v1);

% (25) FOC for labor
(1-alpha)*exp(pm)*exp(Ym)/exp(L)=exp(w);

% (26) FOC for investment
exp(xi)*exp(Q)*(1+exp(v2))=exp(Lam(+1))/exp(Pi(+1))*(1+kappa*exp(xi(+1))*exp(Q(+1))*(1+exp(v2(+1))));

% (27) FOC for fm
exp(v1)=Psi*exp(v2)*exp(pk)+exp(pk);

%%%%%%%%%%%%%%%%%%%% Capital production %%%%%%%%%%%%%
% (28) capital production
exp(Ih)=(1-kI/2*(exp(I)/exp(I(-1))-1)^2)*exp(I);

% (29) FOC for capital
1=exp(pk)*(1-kI/2*(exp(I)/exp(I(-1))-1)^2-kI*(exp(I)/exp(I(-1))-1)*(exp(I)/exp(I(-1))))+exp(Lam(+1))*exp(pk(+1))*kI*(exp(I(+1))/exp(I)-1)*(exp(I(+1))/exp(I))^2;

%%%%%%%%%%%%%%%%%%%% monetary policy %%%%%%%%%%%%%%%%%
% (30) Taylor rule
%RTR=(1-rhor)*log(RTRs)+rhor*RTR(-1)+(1-rhor)*(phipi*(Pi-log(Pis))+phiy*(Y-log(Ys)))+sr*er;

% (31) Rre
exp(Rre)=exp(RTR);

% (32) Rd
exp(Rd)=exp(Rre);

%%%%%%%%%%%%%%%%%%%%% QE %%%%%%%%%%%%%%%%%%%%%%%%%%
% (33) QE
exp(xi)*exp(Q)*fcb+exp(xi)*exp(QB)*exp(bcb)=exp(re);

% (34) fcb
fcb=(1-rhof)*fcbs+rhof*fcb(-1)+sf*ef;

% (35) bcb
exp(bcb)=(1-rhob)*bcbs+rhob*exp(bcb(-1))+sb*eb;

%%%%%%%%%%%%%%%%%%%%% Fiscal policy %%%%%%%%%%%%%%%%%%%%%
% (36) Budget constraint
exp(G)+bG/exp(Pi)=T+exp(xi)*exp(QB)*bG*(1-kappa/exp(Pi));

% (37) Government purchase
G=(1-rhoG)*log(Gs)+rhoG*G(-1)+sG*eG;

%%%%%%%%%%%%%%%%%%%%%%% Exogenous shock%%%%%%%%%%%%%%%%%%%%
% (38) A
A = (1-rhoA)*log(As)+rhoA*A(-1)+sA*eA;

% (39) theta
theta = (1-rhor)*log(thetas)+rhor*theta(-1)+et*st;

%%%%%%%%%%%%%%%%%%%%%%% Market clearing condition %%%%%%%%%%%%%%%%%
% (40) Bond market
exp(fm)=exp(f)+fcb;

% (41) Government Bond market
bG=exp(b)+exp(bcb);

% (42) Resource constraint
exp(Y)=exp(C)+exp(I)+exp(G);

% (43) xi
xi=(1-rhoxi)*log(xis)+rhoxi*xi(-1)+sxi*exi;

% (44) Welfare
welf = log(exp(C) - bh*exp(C(-1))) - Kai*exp(L)^(1+eta)/(1+eta) + beta*welf(+1);

% (45) markup shock
epsi=(1-rhor)*epsis+rhor*epsi(-1)+em;

end;

initval;
RTR=log(1/0.95);
end;


shocks;

var eA=1;
var eG=1;
var ef=1;
var eb=1;
var et=1; 
var er=1;
var exi=1;
var em=1;

end;




planner_objective log(exp(C)) - Kai*exp(L)^(1+eta)/(1+eta);
ramsey_model(instruments=(RTR),planner_discount=beta);
%oo_ramsey_policy_steady_state_file=oo_;
%steady(maxit=1000);


//stoch_simul(order=1,irf=50,ar=0,qz_zero_threshold=1e-8) Y Ym K pk Ih welf; 
%stoch_simul(order=2,irf=0, nomoments,periods=0); 
stoch_simul(order=1,irf=30,periods=500) Y C I;

evaluate_planner_objective;

